function varargout = adjsiza(varargin)
% DESCRIPTION [a,b,...] = adjsiza(a,b,...) 
%  Will extend singleton dimensions on each input argument to match
%  the size of the other arguments. If dimensions are not singletons
%  nor of the same size as the rest of the inputs, an error will occur.
%  It is not an error if any argument is empty. The result is matrices
%  of the same size.
% INPUT
%  any number of matrices
% OUTPUT
%  one matrix for each input all of the same size
% TRY 
%  [a, b] = adjsiza(ones(1,1,2,2), ones(1,2,2,1))
%  [a, b] = adjsiza(ones(1,1,2,2), ones(0,2,2,1))
% SEE ALSO
%  adjsiz, adjsizabdim

% by Magnus Almgren 030724 Major clean up 051230

s = sizes(varargin{1:end}); % the sizes of all inputs in one matrix


% There is a small trick here. max([17 NaN]) evaluates to 17
snan = s;
snan(s==1) = nan; % Singletons are filled with NaN  
sequal = (max(snan,[],1) == min(snan,[],1));
sequal(isnan(max(snan,[],1))) = true; % the case when all are NaNs in second dimension
if ~all(sequal,2) & size(s,1) > 1 % Not equal and more than one matrix
  error('Input arguments mismatch')
end
% Create a matrix to be used by repmat in the for loop
sval = max(snan,[],1); 
sval(isnan(sval)) = 1;
resiz = ones(size(s));
sval = repmat(sval,[size(s,1) 1]);
resiz(s==1) = sval(s==1); % change size of singletons

for j = 1:max(nargout,nargin>1)  % go through all arguments
 % an error if number of input matrices to repmat are less than number of outputs
 if prod(resiz(j,:))~=1 % otherwise repdim consumes unnecessary resources
  varargout{j} = repmat(varargin{j}, resiz(j,:)); % expand size to maxs
 else
  varargout{j} = varargin{j}; 
 end
end
